/**
 * axios-middleware v0.1.2
 * (c) 2018 Émile Bergeron
 * @license MIT
 */
'use strict';

/* eslint-disable class-methods-use-this, no-unused-vars */

/**
 * Default middleware implementation. Extend this class to create a custom middleware.
 */
var HttpMiddleware = function HttpMiddleware () {};

HttpMiddleware.prototype.onRequest = function onRequest (config) {
    return config;
};

/**
 * Default implementation is a noop.
 * @param error
 */
HttpMiddleware.prototype.handleRequestError = function handleRequestError (error) { };

/**
 * Default implementation is the identity function.
 * @param {Object} response
 * @returns {Object}
 */
HttpMiddleware.prototype.onResponse = function onResponse (response) {
    return response;
};

/**
 * Default implementation is a noop.
 * @param error
 */
HttpMiddleware.prototype.handleResponseError = function handleResponseError (error) { };

var HttpMiddlewareService = function HttpMiddlewareService(axios) {
    this.middlewares = [];

    this.setHttp(axios);
};

/**
 *
 * @param {Axios} axios
 */
HttpMiddlewareService.prototype.setHttp = function setHttp (axios) {
        var this$1 = this;

    this.unsetHttp();

    this.http = axios;

    var interceptors = axios.interceptors;

    this._requestInterceptor = interceptors.request.use(
        function (config) { return this$1._onRequest(config); },
        function (error) { return this$1._handleRequestError(error); }
    );
    this._responseInterceptor = interceptors.response.use(
        function (response) { return this$1._onResponse(response); },
        function (error) { return this$1._handleResponseError(error); }
    );
    return this;
};

HttpMiddlewareService.prototype.unsetHttp = function unsetHttp () {
    var http = this.http;
    if (this.http) {
        var interceptors = http.interceptors;
        interceptors.request.eject(this._requestInterceptor);
        interceptors.response.eject(this._responseInterceptor);
    }
};

/**
 * @param {HttpMiddleware} middleware
 * @returns {boolean} true if the middleware is already registered.
 */
HttpMiddlewareService.prototype.has = function has (middleware) {
    return this.middlewares.indexOf(middleware) > -1;
};

/**
 * Add a middleware or an array of middlewares to the stack.
 * @param {HttpMiddleware|Array} middlewares
 */
HttpMiddlewareService.prototype.register = function register (middlewares) {
        var this$1 = this;

    // eslint-disable-next-line no-param-reassign
    if (!Array.isArray(middlewares)) { middlewares = [middlewares]; }

    // Test if middlewares are registered more than once.
    middlewares.forEach(function (middleware) {
        if (this$1.has(middleware)) {
            throw new Error('Middleware already registered');
        }
        this$1.middlewares.push(middleware);
    });
};

/**
 * Remove a middleware from the registered stack.
 * @param {HttpMiddleware} middleware
 */
HttpMiddlewareService.prototype.unregister = function unregister (middleware) {
    var index = this.middlewares.indexOf(middleware);
    if (index > -1) {
        this.middlewares.splice(index, 1);
    }
};

HttpMiddlewareService.prototype.reset = function reset () {
    this.middlewares.length = 0;
};

HttpMiddlewareService.prototype._onRequest = function _onRequest (config) {
    return this.middlewares.reduce(
        function (acc, middleware) { return middleware.onRequest(acc); },
        config);
};

HttpMiddlewareService.prototype._handleRequestError = function _handleRequestError (error) {
    this.middlewares.forEach(function (middleware) { return middleware.handleRequestError(error); });
    return Promise.reject(error);
};

HttpMiddlewareService.prototype._onResponse = function _onResponse (response) {
    return this.middlewares.reduce(
        function (acc, middleware) { return middleware.onResponse(acc); },
        response);
};

HttpMiddlewareService.prototype._handleResponseError = function _handleResponseError (error) {
    this.middlewares.forEach(function (middleware) { return middleware.handleResponseError(error); });
    return Promise.reject(error);
};

var index = {
    HttpMiddleware: HttpMiddleware,
    HttpMiddlewareService: HttpMiddlewareService,
    version: '0.1.2',
};

module.exports = index;
